@@ -287,4 +287,25 @@ lwsac_total_overhead(struct lwsac *head);
 LWS_VISIBLE LWS_EXTERN uint8_t *
 lwsac_scan_extant(struct lwsac *head, uint8_t *find, size_t len, int nul);
 
+/**
+ * lwsac_assert_valid() - checks if check..check + len is a valid pointer into memory owned by ac
+ *
+ * \param ac: the lwsac to confirm with
+ * \param check: the pointer to check
+ * \param len: length in bytes of check area
+ *
+ * Returns 0 if the pointer is in the lwsac, or nonzero if the pointer is
+ * not in a valid region of the lwsac
+ */
+
+LWS_VISIBLE LWS_EXTERN int
+_lwsac_assert_valid(struct lwsac *aco, void *check, size_t len, const char *name_ac, const char *name_blob, const char *filename, int line);
+
+#define __lws_sify(_s) _lws_sify(_s)
+#define _lws_sify(_s) #_s
+
+/* use the below macro to invoke the above function */
+
+#define lwsac_assert_valid(_ac, _check, _len) _lwsac_assert_valid(_ac, _check, _len, __lws_sify(_ac), __lws_sify(_check), __FILE__, __LINE__)
+
 ///@}
