@@ -120,7 +120,7 @@ append_string:
 
 		n = lws_b64_decode_string_len(
 			(const char *)args->jws->map_b64.buf[m],
-			args->jws->map_b64.len[m],
+			(int)args->jws->map_b64.len[m],
 			(char *)args->temp, *args->temp_len);
 		if (n < 0) {
 			lwsl_err("%s: b64 decode failed: in len %d, m %d\n", __func__, (int)args->jws->map_b64.len[m], m);
@@ -129,7 +129,7 @@ append_string:
 
 		args->temp += n;
 		*args->temp_len -= n;
-		args->jws->map.len[m] = n;
+		args->jws->map.len[m] = (unsigned int)n;
 	}
 
 	return 0;
@@ -218,11 +218,11 @@ lws_jws_encode_b64_element(struct lws_jws_map *map, int idx,
 	if (*temp_len < lws_base64_size((int)in_len))
 		return -1;
 
-	n = lws_jws_base64_enc(in, in_len, temp, *temp_len);
+	n = lws_jws_base64_enc(in, in_len, temp, (size_t)*temp_len);
 	if (n < 0)
 		return -1;
 
-	map->len[idx] = n;
+	map->len[idx] = (unsigned int)n;
 	map->buf[idx] = temp;
 
 	*temp_len -= n;
@@ -339,20 +339,24 @@ lws_jws_compact_decode(const char *in, int len, struct lws_jws_map *map,
 		return -1;
 
 	while (m < blocks) {
-		n = lws_b64_decode_string_len(map_b64->buf[m], map_b64->len[m],
-					      out, *out_len);
-		if (n < 0) {
-			lwsl_err("%s: b64 decode failed\n", __func__);
-			return -1;
-		}
-		/* replace the map entry with the decoded content */
-		if (n)
-			map->buf[m] = out;
-		else
-			map->buf[m] = NULL;
-		map->len[m++] = n;
-		out += n;
-		*out_len -= n;
+		if ((int)map_b64->len[m]) {
+			n = lws_b64_decode_string_len(map_b64->buf[m], (int)map_b64->len[m],
+						      out, *out_len);
+			if (n < 0) {
+				lwsl_err("%s: b64 decode failed len %d\n",
+							__func__, (int)map_b64->len[m]);
+				return -1;
+			}
+			/* replace the map entry with the decoded content */
+			if (n)
+				map->buf[m] = out;
+			else
+				map->buf[m] = NULL;
+			map->len[m++] = (unsigned int)n;
+			out += n;
+			*out_len -= n;
+		} else
+			m++;
 
 		if (*out_len < 1)
 			return -1;
@@ -368,15 +372,20 @@ lws_jws_compact_decode_map(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 	int n, m = 0;
 
 	for (n = 0; n < LWS_JWS_MAX_COMPACT_BLOCKS; n++) {
-		n = lws_b64_decode_string_len(map_b64->buf[m], map_b64->len[m],
-					      out, *out_len);
-		if (n < 0) {
-			lwsl_err("%s: b64 decode failed\n", __func__);
-			return -1;
-		}
-		/* replace the map entry with the decoded content */
-		map->buf[m] = out;
-		map->len[m++] = n;
+		if ((int)map_b64->len[m]) {
+			n = lws_b64_decode_string_len(map_b64->buf[m], (int)map_b64->len[m],
+						      out, *out_len);
+			if (n < 0) {
+				lwsl_err("%s: b64 decode failed len %d\n",
+						__func__, (int)map_b64->len[m]);
+
+				return -1;
+			}
+			/* replace the map entry with the decoded content */
+			map->buf[m] = out;
+			map->len[m++] = (unsigned int)n;
+		} else
+			m++;
 		out += n;
 		*out_len -= n;
 
@@ -400,7 +409,7 @@ lws_jws_encode_section(const char *in, size_t in_len, int first, char **p,
 	if (!first)
 		*(*p)++ = '.';
 
-	n = lws_jws_base64_enc(in, in_len, *p, len - 1);
+	n = lws_jws_base64_enc(in, in_len, *p, (unsigned int)len - 1);
 	if (n < 0)
 		return -1;
 
@@ -422,7 +431,7 @@ lws_jws_compact_encode(struct lws_jws_map *map_b64, /* b64-encoded */
 			map_b64->len[n] = 0;
 			continue;
 		}
-		m = lws_jws_base64_enc(map->buf[n], map->len[n], buf, *len);
+		m = lws_jws_base64_enc(map->buf[n], map->len[n], buf, (size_t)*len);
 		if (m < 0)
 			return -1;
 		buf += m;
@@ -461,7 +470,7 @@ lws_jws_sig_confirm(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 	if (!map_b64->buf[LJWS_SIG] && !map->buf[LJWS_UHDR])
 		b = 2;
 
-	if (lws_jws_parse_jose(&jose, map->buf[LJWS_JOSE], map->len[LJWS_JOSE],
+	if (lws_jws_parse_jose(&jose, map->buf[LJWS_JOSE], (int)map->len[LJWS_JOSE],
 			       temp, &temp_len) < 0 || !jose.alg) {
 		lwsl_notice("%s: parse failed\n", __func__);
 		return -1;
@@ -571,7 +580,7 @@ lws_jws_sig_confirm(struct lws_jws_map *map_b64, struct lws_jws_map *map,
 
 		/* 7) Compare the computed and decoded hashes */
 
-		if (lws_timingsafe_bcmp(digest, map->buf[2], h_len)) {
+		if (lws_timingsafe_bcmp(digest, map->buf[2], (uint32_t)h_len)) {
 			lwsl_notice("digest mismatch\n");
 
 			return -1;
@@ -783,13 +792,13 @@ lws_jws_sign_from_b64(struct lws_jose *jose, struct lws_jws *jws,
 			return -1;
 		}
 
-		n = jws->jwk->e[LWS_GENCRYPTO_RSA_KEYEL_N].len;
-		buf = lws_malloc(lws_base64_size(n), "jws sign");
+		n = (int)jws->jwk->e[LWS_GENCRYPTO_RSA_KEYEL_N].len;
+		buf = lws_malloc((unsigned int)lws_base64_size(n), "jws sign");
 		if (!buf)
 			return -1;
 
 		n = lws_genrsa_hash_sign(&rsactx, digest, jose->alg->hash_type,
-					 buf, n);
+					 buf, (unsigned int)n);
 		lws_genrsa_destroy(&rsactx);
 		if (n < 0) {
 			lwsl_err("%s: lws_genrsa_hash_sign failed\n", __func__);
@@ -798,7 +807,7 @@ lws_jws_sign_from_b64(struct lws_jose *jose, struct lws_jws *jws,
 			return -1;
 		}
 
-		n = lws_jws_base64_enc((char *)buf, n, b64_sig, sig_len);
+		n = lws_jws_base64_enc((char *)buf, (unsigned int)n, b64_sig, sig_len);
 		lws_free(buf);
 		if (n < 0) {
 			lwsl_err("%s: lws_jws_base64_enc failed\n", __func__);
@@ -847,14 +856,14 @@ lws_jws_sign_from_b64(struct lws_jose *jose, struct lws_jws *jws,
 			return -1;
 		}
 		m = lws_gencrypto_bits_to_bytes(jose->alg->keybits_fixed) * 2;
-		buf = lws_malloc(m, "jws sign");
+		buf = lws_malloc((unsigned int)m, "jws sign");
 		if (!buf)
 			return -1;
 
 		n = lws_genecdsa_hash_sign_jws(&ecdsactx, digest,
 					       jose->alg->hash_type,
 					       jose->alg->keybits_fixed,
-					       (uint8_t *)buf, m);
+					       (uint8_t *)buf, (unsigned int)m);
 		lws_genec_destroy(&ecdsactx);
 		if (n < 0) {
 			lws_free(buf);
@@ -863,7 +872,7 @@ lws_jws_sign_from_b64(struct lws_jose *jose, struct lws_jws *jws,
 			return -1;
 		}
 
-		n = lws_jws_base64_enc((char *)buf, m, b64_sig, sig_len);
+		n = lws_jws_base64_enc((char *)buf, (unsigned int)m, b64_sig, sig_len);
 		lws_free(buf);
 
 		return n;
@@ -896,29 +905,29 @@ lws_jws_write_flattened_json(struct lws_jws *jws, char *flattened, size_t len)
 	if (len < 1)
 		return 1;
 
-	n += lws_snprintf(flattened + n, len - n , "{\"payload\": \"");
+	n += (unsigned int)lws_snprintf(flattened + n, len - n , "{\"payload\": \"");
 	lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_PYLD],
 			jws->map_b64.len[LJWS_PYLD], len - n);
-	n += strlen(flattened + n);
+	n = n + strlen(flattened + n);
 
-	n += lws_snprintf(flattened + n, len - n , "\",\n \"protected\": \"");
+	n += (unsigned int)lws_snprintf(flattened + n, len - n , "\",\n \"protected\": \"");
 	lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_JOSE],
 			jws->map_b64.len[LJWS_JOSE], len - n);
-	n += strlen(flattened + n);
+	n = n + strlen(flattened + n);
 
 	if (jws->map_b64.buf[LJWS_UHDR]) {
-		n += lws_snprintf(flattened + n, len - n , "\",\n \"header\": ");
+		n += (unsigned int)lws_snprintf(flattened + n, len - n , "\",\n \"header\": ");
 		lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_UHDR],
 				jws->map_b64.len[LJWS_UHDR], len - n);
-		n += strlen(flattened + n);
+		n = n + strlen(flattened + n);
 	}
 
-	n += lws_snprintf(flattened + n, len - n , "\",\n \"signature\": \"");
+	n += (unsigned int)lws_snprintf(flattened + n, len - n , "\",\n \"signature\": \"");
 	lws_strnncpy(flattened + n, jws->map_b64.buf[LJWS_SIG],
 			jws->map_b64.len[LJWS_SIG], len - n);
-	n += strlen(flattened + n);
+	n = n + strlen(flattened + n);
 
-	n += lws_snprintf(flattened + n, len - n , "\"}\n");
+	n += (unsigned int)lws_snprintf(flattened + n, len - n , "\"}\n");
 
 	return (n >= len - 1);
 }
@@ -969,7 +978,7 @@ lws_jwt_signed_validate(struct lws_context *ctx, struct lws_jwk *jwk,
 	 * blocks
 	 */
 
-	n = lws_jws_compact_decode(com, (int)len, &jws.map, &jws.map_b64,
+	n = (size_t)lws_jws_compact_decode(com, (int)len, &jws.map, &jws.map_b64,
 				   temp, &tl);
 	if (n != 3) {
 		lwsl_err("%s: concat_map failed: %d\n", __func__, (int)n);
@@ -984,7 +993,7 @@ lws_jwt_signed_validate(struct lws_context *ctx, struct lws_jwk *jwk,
 	 */
 
 	if (lws_jws_parse_jose(&jose, jws.map.buf[LJWS_JOSE],
-			       jws.map.len[LJWS_JOSE], temp, &tl) < 0) {
+			       (int)jws.map.len[LJWS_JOSE], temp, &tl) < 0) {
 		lwsl_err("%s: JOSE parse failed\n", __func__);
 		goto bail;
 	}
@@ -998,7 +1007,7 @@ lws_jwt_signed_validate(struct lws_context *ctx, struct lws_jwk *jwk,
 	n = strlen(jose.alg->alg);
 
 	do {
-		ts.e = lws_tokenize(&ts);
+		ts.e = (int8_t)lws_tokenize(&ts);
 		if (ts.e == LWS_TOKZE_TOKEN && ts.token_len == n &&
 		    !strncmp(jose.alg->alg, ts.token, ts.token_len))
 			break;
@@ -1039,81 +1048,111 @@ bail:
 	return r;
 }
 
-int
-lws_jwt_sign_compact(struct lws_context *ctx, struct lws_jwk *jwk,
-		     const char *alg, char *out, size_t *out_len, char *temp,
-		     int tl, const char *format, ...)
+static int lws_jwt_vsign_via_info(struct lws_context *ctx, struct lws_jwk *jwk,
+    const struct lws_jwt_sign_info *info, const char *format, va_list ap)
 {
-	int n, r = 1, otl = tl;
+	size_t actual_hdr_len;
 	struct lws_jose jose;
 	struct lws_jws jws;
-	va_list ap;
-	char *q;
+	va_list ap_cpy;
+	int n, r = 1;
+	int otl, tlr;
+	char *p, *q;
 
 	lws_jws_init(&jws, jwk, ctx);
 	lws_jose_init(&jose);
 
-	if (lws_gencrypto_jws_alg_to_definition(alg, &jose.alg)) {
-		lwsl_err("%s: unknown alg %s\n", __func__, alg);
+	otl = tlr = info->tl;
+	p = info->temp;
 
-		goto bail;
+	/*
+	 * We either just use the provided info->jose_hdr, or build a
+	 * minimal header from info->alg
+	 */
+	actual_hdr_len = info->jose_hdr ? info->jose_hdr_len :
+					  10 + strlen(info->alg);
+
+	if (actual_hdr_len > INT_MAX) {
+	  goto bail;
 	}
 
-	/* create JOSE header, also needed for output */
-
-	if (lws_jws_alloc_element(&jws.map, LJWS_JOSE, temp, &tl,
-				  strlen(alg) + 10, 0)) {
+	if (lws_jws_alloc_element(&jws.map, LJWS_JOSE, info->temp, &tlr,
+				  actual_hdr_len, 0)) {
 		lwsl_err("%s: temp space too small\n", __func__);
-		return 1;
+		goto bail;
 	}
 
-	jws.map.len[LJWS_JOSE] = lws_snprintf((char *)jws.map.buf[LJWS_JOSE],
-					      tl, "{\"alg\":\"%s\"}", alg);
+	if (!info->jose_hdr) {
 
-	temp += otl - tl;
-	otl = tl;
+		/* get algorithm from 'alg' string and write minimal JOSE header */
+		if (lws_gencrypto_jws_alg_to_definition(info->alg, &jose.alg)) {
+			lwsl_err("%s: unknown alg %s\n", __func__, info->alg);
 
-	va_start(ap, format);
-	n = vsnprintf(NULL, 0, format, ap);
-	va_end(ap);
-	if (n + 2 >= tl)
+			goto bail;
+		}
+		jws.map.len[LJWS_JOSE] = (uint32_t)lws_snprintf(
+				(char *)jws.map.buf[LJWS_JOSE], (size_t)otl,
+						"{\"alg\":\"%s\"}", info->alg);
+	} else {
+
+		/*
+		 * Get algorithm by parsing the given JOSE header and copy it,
+		 * if it's ok
+		 */
+		if (lws_jws_parse_jose(&jose, info->jose_hdr,
+				       (int)actual_hdr_len, info->temp, &tlr)) {
+			lwsl_err("%s: invalid jose header\n", __func__);
+			goto bail;
+		}
+		tlr = otl;
+		memcpy((char *)jws.map.buf[LJWS_JOSE], info->jose_hdr,
+								actual_hdr_len);
+		jws.map.len[LJWS_JOSE] = (uint32_t)actual_hdr_len;
+		tlr -= (int)actual_hdr_len;
+	}
+
+	p += otl - tlr;
+	otl = tlr;
+
+	va_copy(ap_cpy, ap);
+	n = vsnprintf(NULL, 0, format, ap_cpy);
+	va_end(ap_cpy);
+	if (n + 2 >= tlr)
 		goto bail;
 
-	q = lws_malloc(n + 2, __func__);
+	q = lws_malloc((unsigned int)n + 2, __func__);
 	if (!q)
 		goto bail;
 
-	va_start(ap, format);
-	vsnprintf(q, n + 2, format, ap);
-	va_end(ap);
+	vsnprintf(q, (unsigned int)n + 2, format, ap);
 
 	/* add the plaintext from stdin to the map and a b64 version */
 
 	jws.map.buf[LJWS_PYLD] = q;
-	jws.map.len[LJWS_PYLD] = n;
+	jws.map.len[LJWS_PYLD] = (uint32_t)n;
 
-	if (lws_jws_encode_b64_element(&jws.map_b64, LJWS_PYLD, temp, &tl,
+	if (lws_jws_encode_b64_element(&jws.map_b64, LJWS_PYLD, p, &tlr,
 				       jws.map.buf[LJWS_PYLD],
 				       jws.map.len[LJWS_PYLD]))
 		goto bail1;
 
-	temp += otl - tl;
-	otl = tl;
+	p += otl - tlr;
+	otl = tlr;
 
 	/* add the b64 JOSE header to the b64 map */
 
-	if (lws_jws_encode_b64_element(&jws.map_b64, LJWS_JOSE, temp, &tl,
+	if (lws_jws_encode_b64_element(&jws.map_b64, LJWS_JOSE, p, &tlr,
 				       jws.map.buf[LJWS_JOSE],
 				       jws.map.len[LJWS_JOSE]))
 		goto bail1;
 
-	temp += otl - tl;
-	otl = tl;
+	p += otl - tlr;
+	otl = tlr;
 
 	/* prepare the space for the b64 signature in the map */
 
-	if (lws_jws_alloc_element(&jws.map_b64, LJWS_SIG, temp, &tl,
-				  lws_base64_size(LWS_JWE_LIMIT_KEY_ELEMENT_BYTES),
+	if (lws_jws_alloc_element(&jws.map_b64, LJWS_SIG, p, &tlr,
+				  (size_t)lws_base64_size(LWS_JWE_LIMIT_KEY_ELEMENT_BYTES),
 				  0))
 		goto bail1;
 
@@ -1126,13 +1165,13 @@ lws_jwt_sign_compact(struct lws_context *ctx, struct lws_jwk *jwk,
 		goto bail1;
 
 	/* set the actual b64 signature size */
-	jws.map_b64.len[LJWS_SIG] = n;
+	jws.map_b64.len[LJWS_SIG] = (uint32_t)n;
 
 	/* create the compact JWS representation */
-	if (lws_jws_write_compact(&jws, out, *out_len))
+	if (lws_jws_write_compact(&jws, info->out, *info->out_len))
 		goto bail1;
 
-	*out_len = strlen(out);
+	*info->out_len = strlen(info->out);
 
 	r = 0;
 
@@ -1148,6 +1187,45 @@ bail:
 	return r;
 }
 
+int
+lws_jwt_sign_via_info(struct lws_context *ctx, struct lws_jwk *jwk,
+		     const struct lws_jwt_sign_info *info, const char *format,
+		     ...)
+{
+	int ret;
+	va_list ap;
+
+	va_start(ap, format);
+	ret = lws_jwt_vsign_via_info(ctx, jwk, info, format, ap);
+	va_end(ap);
+
+	return ret;
+}
+
+int
+lws_jwt_sign_compact(struct lws_context *ctx, struct lws_jwk *jwk,
+		     const char *alg, char *out, size_t *out_len, char *temp,
+		     int tl, const char *format, ...)
+{
+	struct lws_jwt_sign_info info = {
+		.alg		= alg,
+		.jose_hdr	= NULL,
+		.out		= out,
+		.out_len	= out_len,
+		.temp		= temp,
+		.tl		= tl
+	};
+	int r = 1;
+	va_list ap;
+
+	va_start(ap, format);
+
+	r = lws_jwt_vsign_via_info(ctx, jwk, &info, format, ap);
+
+	va_end(ap);
+	return r;
+}
+
 int
 lws_jwt_token_sanity(const char *in, size_t in_len,
 		     const char *iss, const char *aud,
@@ -1189,12 +1267,12 @@ lws_jwt_token_sanity(const char *in, size_t in_len,
 	 * ... and not too late for it?
 	 */
 	cp = lws_json_simple_find(in, in_len, "\"exp\":", &len);
-	exp = (unsigned long)atol(cp);
 	if (!cp || (unsigned long)atol(cp) < now) {
 		lwsl_notice("%s: exp fail %lu vs %lu\n", __func__,
 				cp ? (unsigned long)atol(cp) : 0, now);
 		return 1;
 	}
+	exp = (unsigned long)atol(cp);
 
 	/*
 	 * Caller cares about subject?  Then we must have it, and it can't be
