@@ -1,7 +1,7 @@
 /*
  * libwebsockets - small server side websockets and web server implementation
  *
- * Copyright (C) 2010 - 2020 Andy Green <andy@warmcat.com>
+ * Copyright (C) 2010 - 2025 Andy Green <andy@warmcat.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -82,6 +82,7 @@ lejp_construct(struct lejp_ctx *ctx,
 	ctx->path[0] = '\0';
 	ctx->user = user;
 	ctx->line = 1;
+	ctx->flags = 0; /* user may set after construction */
 
 	ctx->pst_sp = 0;
 	ctx->pst[0].callback = callback;
@@ -154,7 +155,7 @@ lejp_check_path_match(struct lejp_ctx *ctx)
 		s = ctx->path_stride;
 
 	/* we only need to check if a match is not active */
-	for (n = 0; !ctx->path_match &&
+	for (n = 0; //!ctx->path_match &&
 	     n < ctx->pst[ctx->pst_sp].count_paths; n++) {
 		ctx->wildcount = 0;
 		p = ctx->path;
@@ -179,7 +180,7 @@ lejp_check_path_match(struct lejp_ctx *ctx)
 			 *  x.*
 			 * if both options are possible
 			 */
-			while (*p && (*p != '.' || !*q))
+			while (*p && ((*p != '.' && *p != '[') || !*q))
 				p++;
 		}
 		if (*p || *q)
@@ -237,7 +238,7 @@ static const char tokens[] = "rue alse ull ";
 int
 lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 {
-	unsigned char c, n, s;
+	unsigned char c, n, s, defer = 0, comp = 0;
 	int ret = LEJP_REJECT_UNKNOWN;
 
 	if (!ctx->sp && !ctx->pst[ctx->pst_sp].ppos)
@@ -274,9 +275,14 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = '[';
 				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = ']';
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
+
+				if (ctx->flags & LEJP_FLAG_FEAT_LEADING_WC)
+					lejp_check_path_match(ctx);
 				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_ARRAY_START))
 					goto reject_callback;
 				ctx->i[ctx->ipos++] = 0;
+				if (ctx->flags & LEJP_FLAG_FEAT_LEADING_WC)
+					lejp_check_path_match(ctx);
 				if (ctx->ipos > LWS_ARRAY_SIZE(ctx->i)) {
 					ret = LEJP_REJECT_MP_DELIM_ISTACK;
 					goto reject;
@@ -287,20 +293,25 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ret = LEJP_REJECT_IDLE_NO_BRACE;
 				goto reject;
 			}
+
+			if (ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES) {
+				/* since insides of {} can have ',', we should
+				 * add an index level so we can count them
+				 */
+				ctx->i[ctx->ipos++] = 0;
+				if (ctx->ipos > LWS_ARRAY_SIZE(ctx->i)) {
+					ret = LEJP_REJECT_MP_DELIM_ISTACK;
+					goto reject;
+				}
+			}
 			if (ctx->pst[ctx->pst_sp].callback(ctx,
 							   LEJPCB_OBJECT_START))
 				goto reject_callback;
 			ctx->st[ctx->sp].s = LEJP_MEMBERS;
 			break;
 		case LEJP_MEMBERS:
-			if (c == '}') {
-				if (ctx->sp >= 1)
-					goto pop_level;
-
-				ctx->st[ctx->sp].s = LEJP_IDLE;
-				ret = LEJP_REJECT_MEMBERS_NO_CLOSE;
-				goto reject;
-			}
+			if (c == '}')
+				goto pop_level;
 			ctx->st[ctx->sp].s = LEJP_M_P;
 			goto redo_character;
 		case LEJP_M_P:
@@ -458,6 +469,16 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->st[ctx->sp].s = LEJP_MP_COMMA_OR_END;
 				c = LEJP_MEMBERS;
 				lejp_check_path_match(ctx);
+				if (ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES) {
+					/* since insides of {} can have ',', we should
+					 * add an index level so we can count them
+					 */
+					ctx->i[ctx->ipos++] = 0;
+					if (ctx->ipos > LWS_ARRAY_SIZE(ctx->i)) {
+						ret = LEJP_REJECT_MP_DELIM_ISTACK;
+						goto reject;
+					}
+				}
 				if (ctx->pst[ctx->pst_sp].callback(ctx,
 							LEJPCB_OBJECT_START))
 					goto reject_callback;
@@ -474,8 +495,12 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = '[';
 				ctx->path[ctx->pst[ctx->pst_sp].ppos++] = ']';
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
+				if (ctx->flags & LEJP_FLAG_FEAT_LEADING_WC)
+					lejp_check_path_match(ctx);
 				if (ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_ARRAY_START))
 					goto reject_callback;
+				if (ctx->flags & LEJP_FLAG_FEAT_LEADING_WC)
+					lejp_check_path_match(ctx);
 				ctx->i[ctx->ipos++] = 0;
 				if (ctx->ipos > LWS_ARRAY_SIZE(ctx->i)) {
 					ret = LEJP_REJECT_MP_DELIM_ISTACK;
@@ -499,7 +524,9 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					ctx->pst[ctx->pst_sp].ppos = (unsigned char)
 							ctx->st[ctx->sp - 1].p;
 					ctx->ipos = (unsigned char)ctx->st[ctx->sp - 1].i;
-				}
+				} else
+					if (ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES)
+						ctx->ipos--;
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
 				if (ctx->path_match &&
 				    ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
@@ -508,6 +535,10 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					 * smaller than the matching point
 					 */
 					ctx->path_match = 0;
+				if (ctx->pst_sp && !ctx->sp)
+					lejp_parser_pop(ctx);
+				if (ctx->flags & LEJP_FLAG_FEAT_LEADING_WC)
+					lejp_check_path_match(ctx);
 				if (ctx->outer_array && !ctx->sp) { /* ended on ] */
 					n = LEJPCB_ARRAY_END;
 					goto completed;
@@ -642,6 +673,11 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 			if (c == ',') {
 				/* increment this stack level's index */
 				ctx->st[ctx->sp].s = LEJP_M_P;
+
+				if (ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES)
+					if (ctx->ipos)
+						ctx->i[ctx->ipos - 1]++;
+
 				if (!ctx->sp) {
 					ctx->pst[ctx->pst_sp].ppos = 0;
 					/*
@@ -661,11 +697,15 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					 */
 					ctx->path_match = 0;
 
+				lejp_check_path_match(ctx);
+
 				if (ctx->st[ctx->sp - 1].s != LEJP_MP_ARRAY_END)
 					break;
 				/* top level is definitely an array... */
-				if (ctx->ipos)
-					ctx->i[ctx->ipos - 1]++;
+				if (!(ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES))
+					if (ctx->ipos)
+						ctx->i[ctx->ipos - 1]++;
+
 				ctx->st[ctx->sp].s = LEJP_MP_VALUE;
 				break;
 			}
@@ -686,7 +726,10 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					ctx->pst[ctx->pst_sp].ppos = (unsigned char)
 							ctx->st[ctx->sp - 1].p;
 					ctx->ipos = (unsigned char)ctx->st[ctx->sp - 1].i;
-				}
+				} else
+					if (ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES)
+						ctx->ipos--;
+
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
 				if (ctx->path_match &&
 				    ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
@@ -701,6 +744,9 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 					goto completed;
 				}
 
+				if (ctx->pst_sp && !ctx->sp)
+					defer = 1;
+
 				/* do LEJP_MP_ARRAY_END processing */
 				goto redo_character;
 			}
@@ -711,8 +757,9 @@ lejp_parse(struct lejp_ctx *ctx, const unsigned char *json, int len)
 			if (!ctx->sp) {
 				n = LEJPCB_OBJECT_END;
 completed:
-				lejp_check_path_match(ctx);
-				if (ctx->pst[ctx->pst_sp].callback(ctx, (char)n) ||
+				ctx->path_match = 0;
+				//lejp_check_path_match(ctx);
+				if ((n && ctx->pst[ctx->pst_sp].callback(ctx, (char)n)) ||
 				    ctx->pst[ctx->pst_sp].callback(ctx,
 							    LEJPCB_COMPLETE))
 					goto reject_callback;
@@ -723,12 +770,20 @@ completed:
 
 			/* pop */
 pop_level:
+			comp = 0;
+			if (!ctx->sp) {
+				comp = 1;
+				goto def_end;
+			}
 			ctx->sp--;
 			if (ctx->sp) {
 				ctx->pst[ctx->pst_sp].ppos = (unsigned char)ctx->st[ctx->sp].p;
 				ctx->ipos = (unsigned char)ctx->st[ctx->sp].i;
-			}
+			} else
+				if (ctx->flags & LEJP_FLAG_FEAT_OBJECT_INDEXES)
+					ctx->ipos--;
 			ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
+
 			if (ctx->path_match &&
 			    ctx->pst[ctx->pst_sp].ppos <= ctx->path_match_len)
 				/*
@@ -736,11 +791,17 @@ pop_level:
 				 * smaller than the matching point
 				 */
 				ctx->path_match = 0;
-
+def_end:
 			lejp_check_path_match(ctx);
 			if (ctx->pst[ctx->pst_sp].callback(ctx,
 							   LEJPCB_OBJECT_END))
 				goto reject_callback;
+			if (comp) {
+				n = 0;
+				goto completed;
+			}
+			if (ctx->pst_sp > 1 && !ctx->sp)
+				lejp_parser_pop(ctx);
 			break;
 
 		case LEJP_MP_ARRAY_END:
@@ -755,15 +816,20 @@ array_end:
 					ctx->pst[ctx->pst_sp].ppos = (unsigned char)
 							ctx->st[ctx->sp - 1].p;
 				ctx->path[ctx->pst[ctx->pst_sp].ppos] = '\0';
+				lejp_check_path_match(ctx);
 				break;
 			}
 			if (c != ']') {
 				ret = LEJP_REJECT_MP_ARRAY_END_MISSING;
 				goto reject;
 			}
-
+			lejp_check_path_match(ctx);
 			ctx->st[ctx->sp].s = LEJP_MP_COMMA_OR_END;
 			ctx->pst[ctx->pst_sp].callback(ctx, LEJPCB_ARRAY_END);
+			if (defer) {
+				lejp_parser_pop(ctx);
+				defer = 0;
+			}
 			break;
 		}
 
@@ -886,3 +952,61 @@ lejp_error_to_string(int e)
 	return parser_errs[e];
 }
 
+int
+lejp_string_unify(struct lejp_ctx *ctx, struct lwsac **ac)
+{
+	char *p;
+
+	if (ctx->su.pieces == 1)
+		return 0;
+
+	ctx->su.fp = lwsac_use(ac, ctx->su.asl + 1u, 512);
+	if (!ctx->su.fp)
+		return 1;
+
+	p = ctx->su.fp;
+	while (ctx->su.sph) {
+		memcpy(p, ctx->su.sph->piece, ctx->su.sph->len);
+		p += ctx->su.sph->len;
+		ctx->su.sph = ctx->su.sph->next;
+	}
+	*p = '\0';
+
+	return 0;
+}
+
+int
+lejp_string_unify_part(struct lejp_ctx *ctx, struct lwsac **ac, char reason)
+{
+	if (reason == LEJPCB_VAL_STR_START) {
+		memset(&ctx->su, 0, sizeof(ctx->su));
+		ctx->su.sp_next = &ctx->su.sph;
+	}
+
+	if (
+	    (reason == LEJPCB_VAL_STR_START && ctx->npos) ||
+	    (reason == LEJPCB_VAL_STR_CHUNK && ctx->npos) ||
+	    (reason == LEJPCB_VAL_STR_END)) {
+		lejp_string_piece_t *s;
+
+		ctx->su.fp = lwsac_use(ac, ctx->npos + 1u, 512);
+		if (!ctx->su.fp)
+			return 1;
+		memcpy(ctx->su.fp, ctx->buf, ctx->npos);
+		ctx->su.fp[ctx->npos] = '\0';
+		s = lwsac_use_zero(ac, sizeof(*s), 512);
+		if (!s)
+			return 1;
+		*ctx->su.sp_next = s;
+		s->piece = ctx->su.fp;
+		s->len = ctx->npos;
+		ctx->su.asl += ctx->npos;
+		ctx->su.sp_next = &s->next;
+
+		if (reason == LEJPCB_VAL_STR_END)
+			ctx->su.sp_next = &ctx->su.sph;
+	}
+
+	return 0;
+}
+
