@@ -23,21 +23,29 @@
  */
 
 #include "private-lib-core.h"
+#include "private-lib-async-dns.h"
 
 #if defined(LWS_WITH_SYS_ASYNC_DNS)
 lws_async_dns_server_check_t
-lws_plat_asyncdns_init(struct lws_context *context, lws_sockaddr46 *sa46)
+lws_plat_asyncdns_init(struct lws_context *context, lws_async_dns_t *dns)
 {
+	lws_sockaddr46 sa46t;
 	uint32_t ipv4;
-	lws_async_dns_server_check_t s = LADNS_CONF_SERVER_CHANGED;
+	lws_async_dns_server_check_t s = LADNS_CONF_SERVER_SAME;
+	lws_async_dns_server_t *dsrv;
 
 	FreeRTOS_GetAddressConfiguration(NULL, NULL, NULL, &ipv4);
 
-	sa46->sa4.sin_family = AF_INET;
-	if (sa46->sa4.sin_addr.s_addr == ipv4)
-		s = LADNS_CONF_SERVER_SAME;
+	memset(&sa46t, 0, sizeof(sa46t));
 
-	sa46->sa4.sin_addr.s_addr = ipv4;
+	sa46t.sa4.sin_family = AF_INET;
+	sa46t.sa4.sin_addr.s_addr = ipv4;
+
+	dsrv = __lws_async_dns_server_find(dns, &sa46t);
+	if (!dsrv) {
+		__lws_async_dns_server_add(dns, &sa46t);
+		s = LADNS_CONF_SERVER_CHANGED;
+	}
 
 	return s;
 }
