@@ -260,7 +260,7 @@ __lws_wsi_server_new(struct lws_vhost *vh, struct lws *parent_wsi,
 	} while (b < 3 && n < sizeof(tmp1) - 2);
 	tmp1[n] = '\0';
 	lws_snprintf(tmp, sizeof(tmp), "h2_sid%u_(%s)", sid, tmp1);
-	wsi = lws_create_new_server_wsi(vh, parent_wsi->tsi, tmp);
+	wsi = lws_create_new_server_wsi(vh, parent_wsi->tsi, LWSLCG_WSI_MUX, tmp);
 	if (!wsi) {
 		lwsl_notice("new server wsi failed (%s)\n", lws_vh_tag(vh));
 		return NULL;
@@ -534,7 +534,7 @@ lws_h2_settings(struct lws *wsi, struct http2_settings *settings,
 		case H2SET_INITIAL_WINDOW_SIZE:
 			if (b > 0x7fffffff) {
 				lws_h2_goaway(nwsi, H2_ERR_FLOW_CONTROL_ERROR,
-					      "Inital Window beyond max");
+					      "Initial Window beyond max");
 				return 1;
 			}
 
@@ -985,11 +985,18 @@ lws_h2_parse_frame_header(struct lws *wsi)
 
 	/* let the network wsi live a bit longer if subs are active */
 
-	if (!wsi->immortal_substream_count)
-		lws_set_timeout(wsi, PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE,
-				wsi->a.vhost->keepalive_timeout ?
-					wsi->a.vhost->keepalive_timeout : 31);
+	if (!wsi->immortal_substream_count) {
+		int ds = lws_wsi_keepalive_timeout_eff(wsi);
 
+		/*
+		 * A short (5s) timeout here affects the reverse proxy if
+		 * the onward box takes a long time to respond, eg to a POST.
+		 * The mount can override the keepalive timeout, eg, to give
+		 * the right behaviour depending on reverse proxy for a particular
+		 * server.
+		 */
+		lws_set_timeout(wsi, PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE, ds);
+	}
 	if (h2n->sid)
 		h2n->swsi = lws_wsi_mux_from_id(wsi, h2n->sid);
 
@@ -1035,10 +1042,15 @@ lws_h2_parse_frame_header(struct lws *wsi)
 					&& wsi->client_h2_alpn
 #endif
 			) {
-				lwsl_notice("ignoring straggling data fl 0x%x\n",
-						h2n->flags);
-				/* ie, IGNORE */
-				h2n->type = LWS_H2_FRAME_TYPE_COUNT;
+				if (h2n->flags & LWS_H2_FLAG_END_STREAM)
+					lwsl_notice("%s: stragging EOS\n", __func__);
+				else {
+					lwsl_wsi_notice(wsi, "ignoring straggling "
+						"DATA (flags 0x%x, length %d)",
+							h2n->flags, (int)h2n->length);
+					/* ie, IGNORE */
+					h2n->type = LWS_H2_FRAME_TYPE_COUNT;
+				}
 			} else {
 				lwsl_info("%s: received %d bytes data for unknown sid %d, highest known %d\n",
 						__func__, (int)h2n->length, (int)h2n->sid, (int)h2n->highest_sid_opened);
@@ -1526,6 +1538,8 @@ lws_h2_parse_end_of_frame(struct lws *wsi)
 			if (wsi->for_ss) {
 				lws_ss_handle_t *h = (lws_ss_handle_t *)lws_get_opaque_user_data(wsi);
 
+				if (!h)
+					return 1;
 				h2n->swsi->for_ss = 1;
 				wsi->for_ss = 0;
 
@@ -1613,7 +1627,7 @@ lws_h2_parse_end_of_frame(struct lws *wsi)
 
 		if (!h2n->swsi->h2.END_HEADERS) {
 			/* we are not finished yet */
-			lwsl_info("witholding http action for continuation\n");
+			lwsl_info("withholding http action for continuation\n");
 			h2n->cont_exp_sid = h2n->sid;
 			h2n->cont_exp = 1;
 			break;
@@ -1969,7 +1983,7 @@ lws_h2_parse_end_of_frame(struct lws *wsi)
 		break;
 
 	case LWS_H2_FRAME_TYPE_GOAWAY:
-		lwsl_notice("GOAWAY: last sid %u, error 0x%08X, string '%s'\n",
+		lwsl_wsi_notice(wsi, "RX GOAWAY: last sid %u, error 0x%08X, string '%s'\n",
 			  (unsigned int)h2n->goaway_last_sid,
 			  (unsigned int)h2n->goaway_err, h2n->goaway_str);
 
@@ -2174,9 +2188,8 @@ lws_h2_parser(struct lws *wsi, unsigned char *in, lws_filepos_t _inlen,
 				 */
 				if (!wsi->immortal_substream_count)
 					lws_set_timeout(wsi,
-					PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE,
-						wsi->a.vhost->keepalive_timeout ?
-					    wsi->a.vhost->keepalive_timeout : 31);
+						PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE,
+						lws_wsi_keepalive_timeout_eff(wsi));
 
 				if (!h2n->swsi)
 					break;
@@ -2203,13 +2216,15 @@ lws_h2_parser(struct lws *wsi, unsigned char *in, lws_filepos_t _inlen,
 					     WSI_TOKEN_HTTP_CONTENT_LENGTH) &&
 				    h2n->swsi->http.rx_content_length &&
 				    h2n->swsi->http.rx_content_remain <
-						     h2n->length && /* last */
+						     h2n->length - h2n->inside && /* last */
 				    h2n->inside < h2n->length) {
 
-					lwsl_warn("%s: %lu %lu %lu %lu\n", __func__,
+					lwsl_warn("%s: rx.cl: %lu, rx.content_remain: %lu, buf left: %lu, "
+						  "h2->inside: %lu, h2->length: %lu\n", __func__,
+						  (unsigned long)h2n->swsi->http.rx_content_length,
 						  (unsigned long)h2n->swsi->http.rx_content_remain,
-						(unsigned long)(lws_ptr_diff_size_t(iend, in) + 1),
-						(unsigned long)h2n->inside, (unsigned long)h2n->length);
+						  (unsigned long)(lws_ptr_diff_size_t(iend, in) + 1),
+						  (unsigned long)h2n->inside, (unsigned long)h2n->length);
 
 					/* unread data in frame */
 					lws_h2_goaway(wsi,
@@ -2359,7 +2374,7 @@ do_windows:
 					 * stream credit to run down until the
 					 * user code deals with it
 					 */
-					lws_h2_update_peer_txcredit(wsi, 0, n);
+					lws_h2_update_peer_txcredit(wsi, (unsigned int)0, n);
 					h2n->swsi->txc.manual = 1;
 				}
 #endif
@@ -2587,6 +2602,8 @@ lws_h2_client_handshake(struct lws *wsi)
 		n--;
 	}
 
+	// lwsl_hexdump_notice(path, (size_t)n);
+
 	if (n && lws_add_http_header_by_token(wsi,
 				WSI_TOKEN_HTTP_COLON_PATH,
 				(unsigned char *)path, n, &p, end))
@@ -2774,6 +2791,17 @@ lws_h2_ws_handshake(struct lws *wsi)
 		}
 	}
 
+#if !defined(LWS_WITHOUT_EXTENSIONS)
+        /*
+         * Figure out which extensions the client has that we want to
+         * enable on this connection, and give him back the list.
+         *
+         * Give him a limited write bugdet
+         */
+        if (lws_extension_server_handshake(wsi, (char **)&p, 192))
+               return -1;
+#endif
+
 	if (lws_finalize_http_header(wsi, &p, end))
 		return -1;
 
@@ -2863,6 +2891,9 @@ lws_read_h2(struct lws *wsi, unsigned char *buf, lws_filepos_t len)
 		 */
 
 		m = lws_h2_parser(wsi, buf, len, &body_chunk_len);
+
+               body_chunk_len &= 0xffffffff; /* attempt workaround for finding len=7167 --> len = 0xffffffff00001bff on lws.org */
+
 		if (m && m != 2) {
 			lwsl_debug("%s: http2_parser bail: %d\n", __func__, m);
 			lws_close_free_wsi(wsi, LWS_CLOSE_STATUS_NOSTATUS,
