@@ -72,6 +72,10 @@ check_extant(struct lws_dll2 *d, void *user)
 	if (wsi->af != a ->af)
 		return 0;
 
+	if (a->info && a->info->vh_listen_sockfd &&
+	    wsi->desc.sockfd != a->info->vh_listen_sockfd)
+		return 0;
+
 	lwsl_notice(" using listen skt from vhost %s\n", wsi->a.vhost->name);
 
 	return 1;
@@ -86,10 +90,10 @@ _lws_vhost_init_server_af(struct vh_sock_args *a)
 {
 	struct lws_context *cx = a->vhost->context;
 	struct lws_context_per_thread *pt;
-	int n, opt = 1, limit = 1;
+	int n, opt = 1, limit = 1, san = 2;
 	lws_sockfd_type sockfd;
 	struct lws *wsi;
-	int m = 0, is;
+	int m = 0, is = 0;
 #if defined(LWS_WITH_IPV6)
 	int value = 1;
 #endif
@@ -104,7 +108,10 @@ _lws_vhost_init_server_af(struct vh_sock_args *a)
 
 deal:
 
-	if (a->vhost->iface) {
+	if (!san--)
+		return -1;
+
+	if (a->vhost->iface && (!a->info || !a->info->vh_listen_sockfd)) {
 
 		/*
 		 * let's check before we do anything else about the disposition
@@ -180,6 +187,11 @@ done_list:
 				LWS_SERVER_OPTION_FAIL_UPON_UNABLE_TO_BIND ?
 				-1 : 1;
 		}
+	} else {
+		if (a->info && a->info->vh_listen_sockfd) {
+			a->vhost->iface = "inherited";
+			a->vhost->listen_port = a->info->port;
+		}
 	}
 
 	(void)n;
@@ -199,7 +211,24 @@ done_list:
 
 	for (m = 0; m < limit; m++) {
 
-		sockfd = lws_fi(&a->vhost->fic, "listenskt") ?
+		if (a->info && a->info->vh_listen_sockfd)
+		{
+#if defined(_WIN32)
+			if (!DuplicateHandle(GetCurrentProcess(),
+					(HANDLE)a->info->vh_listen_sockfd,
+					GetCurrentProcess(), (HANDLE*)&sockfd, 0,
+					FALSE, DUPLICATE_SAME_ACCESS))
+				sockfd = LWS_SOCK_INVALID;
+#else
+#if defined(LWS_PLAT_FREERTOS)
+			sockfd = a->info->vh_listen_sockfd;
+#else
+			sockfd = dup(a->info->vh_listen_sockfd);
+#endif
+#endif
+		}
+		else
+			sockfd = lws_fi(&a->vhost->fic, "listenskt") ?
 					LWS_SOCK_INVALID :
 					socket(a->af, SOCK_STREAM, 0);
 
@@ -245,9 +274,11 @@ done_list:
 		 * There will be a separate ipv4 listen socket if that's
 		 * enabled.
 		 */
-		if (a->af == AF_INET6 &&
+		if (a->af == AF_INET6 && (!a->info || !a->info->vh_listen_sockfd) &&
 		    setsockopt(sockfd, IPPROTO_IPV6, IPV6_V6ONLY,
 			       (const void*)&value, sizeof(value)) < 0) {
+			lwsl_err("ipv6 only failed\n");
+
 			compatible_close(sockfd);
 			return -1;
 		}
@@ -264,35 +295,38 @@ done_list:
 		if (n || cx->count_threads > 1) /* ... also implied by threads > 1 */
 			if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEPORT,
 					(const void *)&opt, sizeof(opt)) < 0) {
-				compatible_close(sockfd);
-				return -1;
+				lwsl_info("reuseport failed\n");
+//				compatible_close(sockfd);
+//				return -1;
 			}
 #endif
 #endif
 		lws_plat_set_socket_options(a->vhost, sockfd, 0);
 
-		is = lws_socket_bind(a->vhost, NULL, sockfd,
-				     a->vhost->listen_port,
-				     a->vhost->iface, a->af);
+		if (!a->info || !a->info->vh_listen_sockfd) {
+			is = lws_socket_bind(a->vhost, NULL, sockfd,
+					     a->vhost->listen_port,
+					     a->vhost->iface, a->af);
 
-		if (is == LWS_ITOSA_BUSY) {
-			/* treat as fatal */
-			compatible_close(sockfd);
+			if (is == LWS_ITOSA_BUSY) {
+				/* treat as fatal */
+				compatible_close(sockfd);
 
-			return -1;
-		}
+				return -1;
+			}
 
-		/*
-		 * There is a race where the network device may come up and then
-		 * go away and fail here.  So correctly handle unexpected failure
-		 * here despite we earlier confirmed it.
-		 */
-		if (is < 0) {
-			lwsl_info("%s: lws_socket_bind says %d\n", __func__, is);
-			compatible_close(sockfd);
-			if (a->vhost->iface)
-				goto deal;
-			return -1;
+			/*
+			 * There is a race where the network device may come up and then
+			 * go away and fail here.  So correctly handle unexpected failure
+			 * here despite we earlier confirmed it.
+			 */
+			if (is < 0) {
+				lwsl_info("%s: lws_socket_bind says %d\n", __func__, is);
+				compatible_close(sockfd);
+				if (a->vhost->iface)
+					goto deal;
+				return -1;
+			}
 		}
 
 		/*
@@ -336,6 +370,8 @@ done_list:
 			goto bail;
 		}
 
+		lws_dll2_remove(&wsi->pre_natal);
+
 		lws_dll2_add_tail(&wsi->listen_list, &a->vhost->listen_wsi);
 		lws_pt_unlock(pt);
 
@@ -372,13 +408,17 @@ done_list:
 			goto bail;
 		}
 
-		if (wsi)
+		if (wsi) {
+			if (a->info && a->info->vh_listen_sockfd)
+				a->vhost->listen_port = a->info->port;
+
 			__lws_lc_tag(a->vhost->context,
 				     &a->vhost->context->lcg[LWSLCG_WSI],
 				     &wsi->lc, "listen|%s|%s|%d",
 				     a->vhost->name,
 				     a->vhost->iface ? a->vhost->iface : "",
 				     (int)a->vhost->listen_port);
+		}
 
 	} /* for each thread able to independently listen */
 
@@ -398,6 +438,7 @@ done_list:
 	return 0;
 
 bail:
+	lwsl_err("%s: bailing\n", __func__);
 	compatible_close(sockfd);
 
 	return -1;
@@ -425,6 +466,11 @@ _lws_vhost_init_server(const struct lws_context_creation_info *info,
 	    vhost->listen_port == CONTEXT_PORT_NO_LISTEN_SERVER)
 		return 0;
 
+	if (info && info->vh_listen_sockfd) {
+		a.af = AF_UNSPEC;
+		goto single;
+	}
+
 	/*
 	 * Let's figure out what AF(s) we want this vhost to listen on.
 	 *
@@ -587,6 +633,7 @@ static const struct lws_mimetype {
 	{ ".xml", "application/xml" },
 	{ ".json", "application/json" },
 	{ ".mjs", "text/javascript" },
+	{ ".wasm", "application/wasm" },
 };
 
 const char *
@@ -702,7 +749,7 @@ lws_http_serve(struct lws *wsi, char *uri, const char *origin,
 		if (wsi->http.fop_fd)
 			lws_vfs_file_close(&wsi->http.fop_fd);
 
-		wsi->http.fop_fd = fops->LWS_FOP_OPEN(wsi->a.context->fops,
+		wsi->http.fop_fd = fops->LWS_FOP_OPEN(fops, wsi->a.context->fops,
 							path, vpath, &fflags);
 		if (!wsi->http.fop_fd) {
 			lwsl_info("%s: Unable to open '%s': errno %d\n",
@@ -957,6 +1004,9 @@ lws_find_mount(struct lws *wsi, const char *uri_ptr, int uri_len)
 			lws_metrics_tag_wsi_add(wsi, "mnt", hm->mountpoint);
 #endif
 
+			if (hm->origin_protocol == LWSMPRO_NO_MOUNT)
+				return NULL;
+
 			if (hm->origin_protocol == LWSMPRO_CALLBACK ||
 			    ((hm->origin_protocol == LWSMPRO_CGI ||
 			     lws_hdr_total_length(wsi, WSI_TOKEN_GET_URI) ||
@@ -1150,6 +1200,22 @@ lws_http_get_uri_and_method(struct lws *wsi, char **puri_ptr, int *puri_len)
 
 #if defined(LWS_WITH_HTTP_BASIC_AUTH)
 
+static int
+lws_authorization_rewrite(struct lws *wsi, const char *name, size_t len)
+{
+	char *p = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_AUTHORIZATION);
+	int fi = wsi->http.ah->frag_index[WSI_TOKEN_HTTP_AUTHORIZATION];
+
+	if (!p)
+		return 1;
+
+	wsi->http.ah->frags[fi].len = (uint16_t)len;
+	strncpy(p, name, (unsigned int)len);
+	p[len] = '\0';
+
+	return 0;
+}
+
 enum lws_check_basic_auth_results
 lws_check_basic_auth(struct lws *wsi, const char *basic_auth_login_file,
 		     unsigned int auth_mode)
@@ -1157,6 +1223,7 @@ lws_check_basic_auth(struct lws *wsi, const char *basic_auth_login_file,
 #if defined(LWS_WITH_FILE_OPS)
 	char b64[160], plain[(sizeof(b64) * 3) / 4], *pcolon;
 	int m, ml, fi, bar;
+	size_t le;
 
 	if (!basic_auth_login_file && auth_mode == LWSAUTHM_DEFAULT)
 		return LCBA_CONTINUE;
@@ -1226,13 +1293,12 @@ lws_check_basic_auth(struct lws *wsi, const char *basic_auth_login_file,
 	 * authorized username
 	 */
 
-	*pcolon = '\0';
-	wsi->http.ah->frags[fi].len = (uint16_t)lws_ptr_diff_size_t(pcolon, &plain[0]);
-	pcolon = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_AUTHORIZATION);
-	strncpy(pcolon, plain, (unsigned int)(ml - 1));
-	pcolon[ml - 1] = '\0';
-	lwsl_info("%s: basic auth accepted for %s\n", __func__,
-		 lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_AUTHORIZATION));
+	le = lws_ptr_diff_size_t(pcolon, plain);
+	plain[le] = '\0';
+	lws_authorization_rewrite(wsi, plain, le);
+
+	lwsl_wsi_info(wsi, "%s: basic auth accepted for '%s'\n", __func__,       
+			lws_hdr_simple_ptr(wsi, WSI_TOKEN_HTTP_AUTHORIZATION));
 
 	return LCBA_CONTINUE;
 #else
@@ -1369,12 +1435,13 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 	}
 
 	i.path = rpath;
-	lwsl_notice("%s: proxied path '%s'\n", __func__, i.path);
+	lwsl_wsi_info(wsi, "proxied path '%s'", i.path);
 
 	/* incoming may be h1 or h2... if he sends h1 HOST, use that
 	 * directly, otherwise we must convert h2 :authority to h1
 	 * host */
 
+#if 0
 	i.host = NULL;
 #if defined(LWS_ROLE_H2)
 	n = lws_hdr_total_length(wsi, WSI_TOKEN_HTTP_COLON_AUTHORITY);
@@ -1387,6 +1454,8 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 		if (n > 0)
 			i.host = lws_hdr_simple_ptr(wsi, WSI_TOKEN_HOST);
 	}
+#endif
+	i.host = i.address;
 
 #if 0
 	if (i.address[0] != '+' ||
@@ -1487,6 +1556,8 @@ lws_http_proxy_start(struct lws *wsi, const struct lws_http_mount *hit,
 	lwsl_info("%s: setting proxy clientside on %s (parent %s)\n",
 		  __func__, lws_wsi_tag(cwsi), lws_wsi_tag(lws_get_parent(cwsi)));
 
+	cwsi->http.mount_specific_keepalive_timeout_secs = (unsigned int)lws_wsi_keepalive_timeout_eff(wsi);
+
 	cwsi->http.proxy_clientside = 1;
 	if (ws) {
 		wsi->proxied_ws_parent = 1;
@@ -1568,8 +1639,6 @@ lws_http_redirect_hit(struct lws_context_per_thread *pt, struct lws *wsi,
 				    lws_hdr_simple_ptr(wsi,
 						WSI_TOKEN_HTTP_COLON_AUTHORITY),
 				    uri_ptr);
-#else
-				;
 #endif
 			} else
 				n = lws_snprintf((char *)end, 256,
@@ -1783,6 +1852,10 @@ lws_http_action(struct lws *wsi)
 
 	/* can we serve it from the mount list? */
 
+	wsi->mount_hit = 0;
+	wsi->http.mount_specific_headers = NULL;
+	wsi->http.mount_specific_keepalive_timeout_secs = 0;
+
 	hit = lws_find_mount(wsi, uri_ptr, uri_len);
 	if (!hit) {
 		/* deferred cleanup and reset to protocols[0] */
@@ -1801,6 +1874,12 @@ lws_http_action(struct lws *wsi)
 		goto after;
 	}
 
+	wsi->mount_hit = 1;
+	wsi->http.mount_specific_headers = hit->headers;
+	wsi->http.mount_specific_keepalive_timeout_secs = hit->keepalive_timeout;
+
+	// lwsl_wsi_notice(wsi, "keepalive_timeout %d ******************\n", hit->keepalive_timeout);
+
 #if defined(LWS_WITH_FILE_OPS)
 	s = uri_ptr + hit->mountpoint_len;
 #endif
@@ -1908,6 +1987,7 @@ lws_http_action(struct lws *wsi)
 			NULL, /* replace with cgi path */
 			NULL
 		};
+		struct lws_cgi_info cgiinfo;
 
 		lwsl_debug("%s: cgi\n", __func__);
 		cmd[0] = hit->origin;
@@ -1916,8 +1996,16 @@ lws_http_action(struct lws *wsi)
 		if (hit->cgi_timeout)
 			n = (unsigned int)hit->cgi_timeout;
 
-		n = (unsigned int)lws_cgi(wsi, cmd, hit->mountpoint_len, (int)n,
-			    hit->cgienv);
+		memset (&cgiinfo, 0, sizeof (cgiinfo));
+		cgiinfo.wsi = wsi;
+		cgiinfo.exec_array = cmd;
+		cgiinfo.script_uri_path_len = hit->mountpoint_len;
+		cgiinfo.timeout_secs = (int)n;
+		cgiinfo.mp_cgienv = hit->cgienv;
+		cgiinfo.chroot_path = hit->cgi_chroot_path;
+		cgiinfo.wd = hit->cgi_wd;
+
+		n = (unsigned int)lws_cgi_via_info(&cgiinfo);
 		if (n) {
 			lwsl_err("%s: cgi failed\n", __func__);
 			return -1;
@@ -1938,6 +2026,7 @@ lws_http_action(struct lws *wsi)
 	wsi->cache_secs = (unsigned int)hit->cache_max_age;
 	wsi->cache_reuse = hit->cache_reusable;
 	wsi->cache_revalidate = hit->cache_revalidate;
+	wsi->cache_no = hit->cache_no;
 	wsi->cache_intermediaries = hit->cache_intermediaries;
 
 #if defined(LWS_WITH_FILE_OPS)
@@ -1955,7 +2044,6 @@ lws_http_action(struct lws *wsi)
 			const struct lws_protocols *pp =
 					lws_vhost_name_to_protocol(
 						wsi->a.vhost, hit->protocol);
-
 			/* coverity */
 			if (!pp)
 				return 1;
@@ -2122,7 +2210,8 @@ lws_confirm_host_header(struct lws *wsi)
 		if (e != LWS_TOKZE_ENDED)
 			goto bad_format;
 
-	if (wsi->a.vhost->listen_port != port) {
+	if (wsi->a.vhost->listen_port != port &&
+		wsi->a.vhost->listen_port != CONTEXT_PORT_NO_LISTEN_SERVER) {
 		lwsl_info("%s: host port %d mismatches vhost port %d\n",
 			  __func__, port, wsi->a.vhost->listen_port);
 		return 1;
@@ -2377,49 +2466,49 @@ raw_transition:
 						HTTP_STATUS_FORBIDDEN, NULL) ||
 				    lws_http_transaction_completed(wsi))
 					goto bail_nuke_ah;
-			}
+			} else {
+				n = user_callback_handle_rxflow(wsi->a.protocol->callback,
+						wsi, LWS_CALLBACK_HTTP_CONFIRM_UPGRADE,
+						wsi->user_space, (char *)up, 0);
 
-			n = user_callback_handle_rxflow(wsi->a.protocol->callback,
-					wsi, LWS_CALLBACK_HTTP_CONFIRM_UPGRADE,
-					wsi->user_space, (char *)up, 0);
+				/* just hang up? */
 
-			/* just hang up? */
-
-			if (n < 0)
-				goto bail_nuke_ah;
-
-			/* callback returned headers already, do t_c? */
-
-			if (n > 0) {
-				if (lws_http_transaction_completed(wsi))
+				if (n < 0)
 					goto bail_nuke_ah;
 
-				/* continue on */
+				/* callback returned headers already, do t_c? */
 
-				return 0;
-			}
+				if (n > 0) {
+				    if (lws_http_transaction_completed(wsi))
+					goto bail_nuke_ah;
 
-			/* callback said 0, it was allowed */
+				    /* continue on */
 
-			if (wsi->a.vhost->options &
-			    LWS_SERVER_OPTION_VHOST_UPG_STRICT_HOST_CHECK &&
-			    lws_confirm_host_header(wsi))
-				goto bail_nuke_ah;
+				    return 0;
+				}
 
-			if (!strcasecmp(up, "websocket")) {
+				/* callback said 0, it was allowed */
+
+				if (wsi->a.vhost->options &
+					LWS_SERVER_OPTION_VHOST_UPG_STRICT_HOST_CHECK &&
+					lws_confirm_host_header(wsi))
+				    goto bail_nuke_ah;
+
+				if (!strcasecmp(up, "websocket")) {
 #if defined(LWS_ROLE_WS)
-				lws_metrics_tag_wsi_add(wsi, "upg", "ws");
-				lwsl_info("Upgrade to ws\n");
-				goto upgrade_ws;
+					lws_metrics_tag_wsi_add(wsi, "upg", "ws");
+					lwsl_info("Upgrade to ws\n");
+					goto upgrade_ws;
 #endif
-			}
+				}
 #if defined(LWS_WITH_HTTP2)
-			if (!strcasecmp(up, "h2c")) {
-				lws_metrics_tag_wsi_add(wsi, "upg", "h2c");
-				lwsl_info("Upgrade to h2c\n");
-				goto upgrade_h2c;
-			}
+				if (!strcasecmp(up, "h2c")) {
+				    lws_metrics_tag_wsi_add(wsi, "upg", "h2c");
+				    lwsl_info("Upgrade to h2c\n");
+				    goto upgrade_h2c;
+				}
 #endif
+			}
 		}
 
 		/* no upgrade ack... he remained as HTTP */
@@ -2520,8 +2609,6 @@ bail_nuke_ah:
 int LWS_WARN_UNUSED_RESULT
 lws_http_transaction_completed(struct lws *wsi)
 {
-	int n;
-
 	if (wsi->http.cgi_transaction_complete)
 		return 0;
 
@@ -2653,10 +2740,17 @@ lws_http_transaction_completed(struct lws *wsi)
 		lws_vfs_file_close(&wsi->http.fop_fd);
 #endif
 
-	n = NO_PENDING_TIMEOUT;
-	if (wsi->a.vhost->keepalive_timeout)
-		n = PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE;
-	lws_set_timeout(wsi, (enum pending_timeout)n, wsi->a.vhost->keepalive_timeout);
+	{
+		enum pending_timeout ept = NO_PENDING_TIMEOUT;
+
+		if (wsi->a.vhost->keepalive_timeout
+#if defined(LWS_WITH_SERVER)
+			|| wsi->http.mount_specific_keepalive_timeout_secs
+#endif
+   	   )
+		ept = PENDING_TIMEOUT_HTTP_KEEPALIVE_IDLE;
+		lws_set_timeout(wsi, ept, lws_wsi_keepalive_timeout_eff(wsi));
+	}
 
 	/*
 	 * We already know we are on http1.1 / keepalive and the next thing
@@ -2758,7 +2852,7 @@ lws_serve_http_file(struct lws *wsi, const char *file, const char *content_type,
 	if (!wsi->http.fop_fd) {
 		fops = lws_vfs_select_fops(wsi->a.context->fops, file, &vpath);
 		fflags |= lws_vfs_prepare_flags(wsi);
-		wsi->http.fop_fd = fops->LWS_FOP_OPEN(wsi->a.context->fops,
+		wsi->http.fop_fd = fops->LWS_FOP_OPEN(fops, wsi->a.context->fops,
 							file, vpath, &fflags);
 		if (!wsi->http.fop_fd) {
 			lwsl_info("%s: Unable to open: '%s': errno %d\n",
@@ -2959,19 +3053,22 @@ lws_serve_http_file(struct lws *wsi, const char *file, const char *content_type,
 		}
 	}
 
-	if (wsi->cache_secs && wsi->cache_reuse) {
+	if (wsi->cache_no) {
+		cc = cache_control;
+		cclen = sprintf(cache_control, "no-cache");
+	}
+	else if (wsi->cache_secs && wsi->cache_reuse) {
 		if (!wsi->cache_revalidate) {
 			cc = cache_control;
 			cclen = sprintf(cache_control, "%s, max-age=%u",
-				    intermediates[wsi->cache_intermediaries],
-				    wsi->cache_secs);
+                         intermediates[wsi->cache_intermediaries],
+                         wsi->cache_secs);
 		} else {
 			cc = cache_control;
 			cclen = sprintf(cache_control,
-					"must-revalidate, %s, max-age=%u",
-                                intermediates[wsi->cache_intermediaries],
-                                                    wsi->cache_secs);
-
+                         "must-revalidate, %s, max-age=%u",
+                         intermediates[wsi->cache_intermediaries],
+                         wsi->cache_secs);
 		}
 	}
 
@@ -3299,12 +3396,17 @@ all_sent:
 					 * state, not the root connection at the
 					 * network level
 					 */
+
 					if (wsi->mux_substream)
 						return 1;
 					else
 						return -1;
 				}
 
+			if (wsi->http.ah)
+				lws_header_table_reset(wsi, 0);
+
+
 			return 1;  /* >0 indicates completed */
 		}
 		/*
