@@ -24,7 +24,7 @@
 
 #include "private-lib-core.h"
 
-static int
+static lws_handling_result_t
 rops_handle_POLLIN_mqtt(struct lws_context_per_thread *pt, struct lws *wsi,
 			   struct lws_pollfd *pollfd)
 {
@@ -170,7 +170,6 @@ drain:
 	}
 
 	ebuf.token = NULL;
-	ebuf.len = 0;
 
 	pending = (unsigned int)lws_ssl_pending(wsi);
 	if (pending) {
@@ -272,7 +271,7 @@ rops_client_bind_mqtt(struct lws *wsi, const struct lws_client_connect_info *i)
 	return 1; /* matched */
 }
 
-static int
+static lws_handling_result_t
 rops_handle_POLLOUT_mqtt(struct lws *wsi)
 {
 	struct lws **wsi2;
@@ -552,6 +551,16 @@ rops_callback_on_writable_mqtt(struct lws *wsi)
 			)
 		return 1;
 
+	if (lws_check_opt(wsi->a.context->options, LWS_SERVER_OPTION_LIBUV)) {
+		if (network_wsi->mux_substream)
+			network_wsi->mux_substream = 0;
+
+		lws_start_foreach_ll(struct lws *, w, network_wsi->mux.child_list) {
+			if (!w->mux.requested_POLLOUT)
+				w->mux.requested_POLLOUT = 1;
+		} lws_end_foreach_ll(w, mux.sibling_list);
+	}
+
 	return 0;
 }
 
@@ -562,6 +571,11 @@ rops_close_kill_connection_mqtt(struct lws *wsi, enum lws_close_status reason)
 			lws_wsi_tag(wsi),
 			lws_wsi_tag(wsi->mux.parent_wsi), wsi->mux.child_list);
 	//lws_wsi_mux_dump_children(wsi);
+	if (lws_check_opt(wsi->a.context->options, LWS_SERVER_OPTION_LIBUV)) {
+		struct lws *nwsi = lws_get_network_wsi(wsi);
+		if (!nwsi->mux_substream)
+			nwsi->mux_substream = 1;
+	}
 
 	if (wsi->mux_substream
 #if defined(LWS_WITH_CLIENT)
