@@ -24,6 +24,27 @@
 
 #include <private-lib-core.h>
 
+#if defined(LWS_WITH_HTTP_PROXY)
+static void
+lws_ws_proxy_est_cb(lws_sorted_usec_list_t *sul)
+{
+	struct lws *wsi = lws_container_of(sul, struct lws, sul_ws_proxy_est);
+
+	lwsi_set_state(wsi, LRS_ESTABLISHED);
+
+	if (wsi->a.protocol->callback)
+		if (wsi->a.protocol->callback(wsi, LWS_CALLBACK_ESTABLISHED,
+					    wsi->user_space,
+#ifdef LWS_WITH_TLS
+					    wsi->tls.ssl,
+#else
+					    NULL,
+#endif
+					    wsi->h2_stream_carries_ws))
+			lws_wsi_close(wsi, LWS_TO_KILL_ASYNC);
+}
+#endif
+
 #define LWS_CPYAPP(ptr, str) { strcpy(ptr, str); ptr += strlen(str); }
 
 /*
@@ -840,7 +861,10 @@ lws_server_init_wsi_for_ws(struct lws *wsi)
 {
 	int n;
 
-	lwsi_set_state(wsi, LRS_ESTABLISHED);
+#if defined(LWS_WITH_HTTP_PROXY)
+	if (!wsi->proxied_ws_parent) 
+		lwsi_set_state(wsi, LRS_ESTABLISHED);
+#endif
 
 	/*
 	 * create the frame buffer for this connection according to the
@@ -861,7 +885,15 @@ lws_server_init_wsi_for_ws(struct lws *wsi)
 
 	/* notify user code that we're ready to roll */
 
-	if (wsi->a.protocol->callback)
+	if (wsi->a.protocol->callback) {
+#if defined(LWS_WITH_HTTP_PROXY)
+		if (wsi->proxied_ws_parent) {
+			lws_sul_schedule(wsi->a.context, wsi->tsi,
+					 &wsi->sul_ws_proxy_est,
+					 lws_ws_proxy_est_cb, 5000);
+			goto validity;
+		}
+#endif
 		if (wsi->a.protocol->callback(wsi, LWS_CALLBACK_ESTABLISHED,
 					    wsi->user_space,
 #ifdef LWS_WITH_TLS
@@ -871,6 +903,10 @@ lws_server_init_wsi_for_ws(struct lws *wsi)
 #endif
 					    wsi->h2_stream_carries_ws))
 			return 1;
+	}
+#if defined(LWS_WITH_HTTP_PROXY)
+validity:
+#endif
 
 	lws_validity_confirmed(wsi);
 	lwsl_debug("ws established\n");
@@ -879,6 +915,11 @@ lws_server_init_wsi_for_ws(struct lws *wsi)
 }
 
 
+int
+lws_ws_sending_multifragment(struct lws *wsi)
+{
+	return wsi->ws->last_valid && !wsi->ws->last_fin;
+}
 
 int
 lws_is_final_fragment(struct lws *wsi)
@@ -1228,6 +1269,9 @@ drain:
 
 	pending = (unsigned int)lws_ssl_pending(wsi);
 
+	if (!pending && lws_buflist_next_segment_len(&wsi->buflist, NULL))
+		return LWS_HPI_RET_HANDLED;
+
 #if defined(LWS_WITH_CLIENT)
 	if (!pending && (wsi->flags & LCCSCF_PRIORITIZE_READS) &&
 	    lws_buflist_total_len(&wsi->buflist))
@@ -1791,12 +1835,37 @@ rops_write_role_protocol_ws(struct lws *wsi, unsigned char *buf, size_t len,
 		switch ((*wp) & 0xf) {
 		case LWS_WRITE_TEXT:
 			n = LWSWSOPC_TEXT_FRAME;
+			if (wsi->ws->last_valid && !wsi->ws->last_fin) {
+				lwsl_wsi_err(wsi, "Sending TEXT after previous frame that lacked FIN");
+				assert(0);
+			}
+			wsi->ws->last_valid = 1;
+			wsi->ws->last_opcode = (uint8_t)n;
+			wsi->ws->last_fin = !((*wp) & LWS_WRITE_NO_FIN);
 			break;
 		case LWS_WRITE_BINARY:
 			n = LWSWSOPC_BINARY_FRAME;
+			if (wsi->ws->last_valid && !wsi->ws->last_fin) {
+				lwsl_wsi_err(wsi, "Sending BINARY after previous frame that lacked FIN");
+				assert(0);
+			}
+			wsi->ws->last_valid = 1;
+			wsi->ws->last_opcode = (uint8_t)n;
+			wsi->ws->last_fin = !((*wp) & LWS_WRITE_NO_FIN);
 			break;
 		case LWS_WRITE_CONTINUATION:
 			n = LWSWSOPC_CONTINUATION;
+			if (wsi->ws->last_valid && wsi->ws->last_fin) {
+				lwsl_wsi_err(wsi, "Sending CONTINUATION after previous frame that had FIN");
+				assert(0);
+			}
+			if (!wsi->ws->last_valid) {
+				lwsl_wsi_err(wsi, "Sending CONTINUATION as first frame");
+				assert(0);
+			}
+			wsi->ws->last_valid = 1;
+			wsi->ws->last_opcode = (uint8_t)n;
+			wsi->ws->last_fin = !((*wp) & LWS_WRITE_NO_FIN);
 			break;
 
 		case LWS_WRITE_CLOSE:
@@ -2079,18 +2148,24 @@ rops_issue_keepalive_ws(struct lws *wsi, int isvalid)
 						     encapsulation_parent(wsi);
 
 		assert(enc);
+		lwsl_wsi_info(wsi, "trying to do keepalive on h2 wrapper around ws");
 		if (lws_rops_func_fidx(enc->role_ops, LWS_ROPS_issue_keepalive).
-						  issue_keepalive(enc, isvalid))
+						  issue_keepalive(enc, isvalid)) {
+			lwsl_wsi_err(wsi, "FAILED to keep h2 wrapper for ws alive");
 			return 1;
+		}
 	}
 #endif
 
-	if (isvalid)
+	if (isvalid) {
+		lwsl_wsi_info(wsi, "confirming validity");
 		_lws_validity_confirmed_role(wsi);
-	else {
+	} else {
 		us = (uint64_t)lws_now_usecs();
 		memcpy(&wsi->ws->ping_payload_buf[LWS_PRE], &us, 8);
 		wsi->ws->send_check_ping = 1;
+		lwsl_wsi_info(wsi, "requesting send ping on ws");
+
 		lws_callback_on_writable(wsi);
 	}
 
