@@ -35,7 +35,7 @@ lws_adns_parse_label(const uint8_t *pkt, int len, const uint8_t *ls, int budget,
 	const uint8_t *e = pkt + len, *ols = ls;
 	char pointer = 0, first = 1;
 	uint8_t ll;
-	int n;
+	int n, readsize = 0;
 
 	if (len < DHO_SIZEOF || len > 1500)
 		return -1;
@@ -93,7 +93,7 @@ again1:
 		return -1;
 	}
 
-	if ((unsigned int)ll + 2 > dl) {
+	if ((unsigned int)(ll + 2 + readsize) > dl) {
 		lwsl_notice("%s: qname too large\n", __func__);
 
 		return -1;
@@ -106,6 +106,7 @@ again1:
 	(*dest)[ll + 1] = '\0';
 	*dest += ll + 1;
 	ls += ll;
+	readsize += ll + 1;
 
 	if (pointer) {
 		if (*ls)
@@ -575,7 +576,7 @@ lws_adns_parse_udp(lws_async_dns_t *dns, const uint8_t *pkt, size_t len)
 	n = 1 << (lws_ser_ru16be(pkt + DHO_TID) & 1);
 	if (q->responded & n) {
 		lwsl_notice("%s: dup\n", __func__);
-		goto fail_out;
+		return;
 	}
 
 	q->responded = (uint8_t)(q->responded | n);
