@@ -24,8 +24,12 @@
  */
 
 #if !defined (LWS_PLUGIN_STATIC)
+#if !defined(LWS_DLL)
 #define LWS_DLL
+#endif
+#if !defined(LWS_INTERNAL)
 #define LWS_INTERNAL
+#endif
 #include <libwebsockets.h>
 #endif
 
@@ -82,7 +86,7 @@ __mirror_rxflow_instance(struct mirror_instance *mi, int enable)
 		lws_rx_flow_control(pss->wsi, enable);
 	} lws_end_foreach_ll(pss, same_mi_pss_list);
 
-	mi->rx_enabled = enable;
+	mi->rx_enabled = (char)enable;
 }
 
 /*
@@ -220,12 +224,12 @@ callback_lws_mirror(struct lws *wsi, enum lws_callback_reasons reason,
 		 * mirror instance name... defaults to "", but if URL includes
 		 * "?mirror=xxx", will be "xxx"
 		 */
-		name[0] = '\0';
-		if (!lws_get_urlarg_by_name(wsi, "mirror", name,
-					   sizeof(name) - 1))
+
+		if (lws_get_urlarg_by_name_safe(wsi, "mirror", name,
+					        sizeof(name) - 1) < 0) {
 			lwsl_debug("get urlarg failed\n");
-		if (strchr(name, '='))
-			pn = strchr(name, '=') + 1;
+			name[0] = '\0';
+		}
 
 		//lwsl_notice("%s: mirror name '%s'\n", __func__, pn);
 
@@ -348,6 +352,8 @@ bail1:
 			v = (struct per_vhost_data__lws_mirror *)
 				lws_protocol_vh_priv_get(lws_get_vhost(wsi),
 							 lws_get_protocol(wsi));
+			if (!v)
+				return 0;
 			lws_pthread_mutex_init(&v->lock);
 		}
 		break;
@@ -478,7 +484,7 @@ done2:
 
 #if !defined (LWS_PLUGIN_STATIC)
 
-static const struct lws_protocols protocols[] = {
+LWS_VISIBLE const struct lws_protocols lws_mirror_protocols[] = {
 	LWS_PLUGIN_PROTOCOL_MIRROR
 };
 
@@ -486,11 +492,12 @@ LWS_VISIBLE const lws_plugin_protocol_t lws_mirror = {
 	.hdr = {
 		"lws mirror",
 		"lws_protocol_plugin",
+		LWS_BUILD_HASH,
 		LWS_PLUGIN_API_MAGIC
 	},
 
-	.protocols = protocols,
-	.count_protocols = LWS_ARRAY_SIZE(protocols),
+	.protocols = lws_mirror_protocols,
+	.count_protocols = LWS_ARRAY_SIZE(lws_mirror_protocols),
 	.extensions = NULL,
 	.count_extensions = 0,
 };
