@@ -23,8 +23,12 @@
  */
 
 #if !defined (LWS_PLUGIN_STATIC)
+#if !defined(LWS_DLL)
 #define LWS_DLL
+#endif
+#if !defined(LWS_INTERNAL)
 #define LWS_INTERNAL
+#endif
 #include <libwebsockets.h>
 #endif
 
@@ -158,7 +162,7 @@ flow_control(struct conn *conn, int side, int enable)
 	if (lws_rx_flow_control(conn->wsi[side], enable))
 		return 1;
 
-	conn->rx_enabled[side] = enable;
+	conn->rx_enabled[side] = (char)enable;
 	lwsl_info("%s: %s side: %s\n", __func__, side ? "ONW" : "ACC",
 		  enable ? "rx enabled" : "rx flow controlled");
 
@@ -187,11 +191,13 @@ callback_raw_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 	case LWS_CALLBACK_PROTOCOL_INIT:
 		vhd = lws_protocol_vh_priv_zalloc(lws_get_vhost(wsi),
 				lws_get_protocol(wsi), sizeof(struct raw_vhd));
+		if (!vhd)
+			return 0;
 		if (lws_pvo_get_str(in, "onward", &cp)) {
-			lwsl_err("%s: vh %s: pvo 'onward' required\n", __func__,
+			lwsl_warn("%s: vh %s: pvo 'onward' required\n", __func__,
 				 lws_get_vhost_name(lws_get_vhost(wsi)));
 
-			return -1;
+			return 0;
 		}
 		lws_tokenize_init(&ts, cp, LWS_TOKENIZE_F_DOT_NONTERM |
 					   LWS_TOKENIZE_F_MINUS_NONTERM |
@@ -223,7 +229,7 @@ callback_raw_proxy(struct lws *wsi, enum lws_callback_reasons reason,
 				e = lws_tokenize(&ts);
 				if (e != LWS_TOKZE_INTEGER)
 					goto bad_onward;
-				vhd->port = atoi(ts.token);
+				vhd->port = (uint16_t)atoi(ts.token);
 				e = lws_tokenize(&ts);
 			}
 			if (e != LWS_TOKZE_ENDED)
@@ -298,7 +304,7 @@ bad_onward:
 			lwsl_notice("OOM: dropping\n");
 			return -1;
 		}
-		pkt.len = len;
+		pkt.len = (uint32_t)len;
 		pkt.ticket = conn->ticket_next++;
 
 		memcpy(pkt.payload, in, len);
@@ -455,7 +461,7 @@ bad_onward:
 			lwsl_notice("OOM: dropping\n");
 			return -1;
 		}
-		pkt.len = len;
+		pkt.len = (uint32_t)len;
 		pkt.ticket = conn->ticket_next++;
 
 		memcpy(pkt.payload, in, len);
@@ -558,7 +564,7 @@ bad_onward:
 
 #if !defined (LWS_PLUGIN_STATIC)
 
-static const struct lws_protocols protocols[] = {
+LWS_VISIBLE const struct lws_protocols lws_raw_proxy_protocols[] = {
 	LWS_PLUGIN_PROTOCOL_RAW_PROXY
 };
 
@@ -566,11 +572,12 @@ LWS_VISIBLE const lws_plugin_protocol_t lws_raw_proxy = {
 	.hdr = {
 		"raw proxy",
 		"lws_protocol_plugin",
+		LWS_BUILD_HASH,
 		LWS_PLUGIN_API_MAGIC
 	},
 
-	.protocols = protocols,
-	.count_protocols = LWS_ARRAY_SIZE(protocols),
+	.protocols = lws_raw_proxy_protocols,
+	.count_protocols = LWS_ARRAY_SIZE(lws_raw_proxy_protocols),
 	.extensions = NULL,
 	.count_extensions = 0,
 };
